from .core.element import HTMLElement
from .tags.form import *
from .tags.html import *
from .tags.layout import *
from .tags.lists import *
from .tags.media import *
from .tags.table import *
from .tags.text import *


import unittest

from ydnatl.core.element import HTMLElement
from ydnatl.tags.layout import Div
from ydnatl.tags.text import Span, Paragraph


class TestHTMLElement(unittest.TestCase):

    def test_prepend(self):
        """Test the prepend() method."""
        element = HTMLElement(tag="div")
        div = HTMLElement(tag="div")
        element.prepend(div)
        self.assertEqual(len(element.children), 1)
        self.assertEqual(str(element.children[0]), "<div></div>")

    def test_append(self):
        """Test the append() method."""
        element = HTMLElement(tag="div")
        div = HTMLElement(tag="div")
        element.append(div)
        self.assertEqual(len(element.children), 1)
        self.assertEqual(str(element.children[0]), "<div></div>")

    def test_filter(self):
        """Test the filter() method."""
        element = HTMLElement(tag="div")
        span = Span("Child 1")
        p = Paragraph("Child 2")
        div = Div("Child 3")
        element.append(span, p, div)

        filtered = list(element.filter(lambda x: x.tag == "span"))
        self.assertEqual(len(filtered), 1)
        self.assertEqual(str(filtered[0]), "<span>Child 1</span>")

    def test_remove_all(self):
        """Test the remove_all() method."""
        element = HTMLElement(tag="div")
        child1 = HTMLElement(tag="span")
        child2 = HTMLElement(tag="p")
        child3 = HTMLElement(tag="div")
        element.append(child1, child2, child3)

        element.remove_all(lambda x: x.tag == "span")
        self.assertEqual(len(element.children), 2)
        self.assertEqual(str(element.children[0]), "<p></p>")
        self.assertEqual(str(element.children[1]), "<div></div>")

    def test_clear(self):
        """Test the clear() method."""
        element = HTMLElement(tag="div")
        element.append(HTMLElement(tag="span"), HTMLElement(tag="p"))
        element.clear()
        self.assertEqual(len(element.children), 0)

    def test_pop(self):
        """Test the pop() method."""
        element = HTMLElement(tag="div")
        child1 = HTMLElement(tag="span")
        child2 = HTMLElement(tag="p")
        element.append(child1, child2)

        popped = element.pop(0)
        self.assertEqual(str(popped), "<span></span>")
        self.assertEqual(len(element.children), 1)
        self.assertEqual(str(element.children[0]), "<p></p>")

    def test_first_last(self):
        """Test the first() and last() methods."""
        element = HTMLElement(tag="div")
        child1 = HTMLElement(tag="span")
        child2 = HTMLElement(tag="p")
        element.append(child1, child2)

        self.assertEqual(str(element.first()), "<span></span>")
        self.assertEqual(str(element.last()), "<p></p>")

    def test_add_remove_attribute(self):
        """Test the add_attribute() and remove_attribute() methods."""
        element = HTMLElement(tag="div")
        element.add_attribute("id", "my-div")
        self.assertEqual(element.attributes, {"id": "my-div"})

        element.remove_attribute("id")
        self.assertEqual(element.attributes, {})

    def test_get_has_attribute(self):
        """Test the get_attribute() and has_attribute() methods."""
        element = HTMLElement(tag="div", id="my-div", class_name="container")
        self.assertEqual(element.get_attribute("id"), "my-div")
        self.assertEqual(element.get_attribute("class_name"), "container")
        self.assertTrue(element.has_attribute("id"))
        self.assertFalse(element.has_attribute("style"))

    def test_generate_id(self):
        """Test the generate_id() method."""
        element = HTMLElement(tag="div")
        element.generate_id()
        self.assertTrue(element.has_attribute("id"))
        self.assertTrue(element.get_attribute("id").startswith("el-"))

    def test_clone(self):
        """Test the clone() method."""
        element = HTMLElement(tag="div", id="my-div", text="Hello")
        child = HTMLElement(tag="span")
        element.append(child)

        cloned = element.clone()
        self.assertEqual(str(cloned), str(element))
        self.assertIsNot(cloned, element)
        self.assertIsNot(cloned.children[0], child)

    def test_find_by_attribute(self):
        """Test the find_by_attribute() method."""
        element = HTMLElement(tag="div")
        child1 = HTMLElement(tag="span", id="child1")
        child2 = HTMLElement(tag="p", id="child2")
        child3 = HTMLElement(tag="div", id="child3")
        element.append(child1, child2, child3)

        found = element.find_by_attribute("id", "child2")
        self.assertEqual(str(found), '<p id="child2"></p>')

        nested_child = HTMLElement(tag="span", id="nested")
        child3.append(nested_child)
        found_nested = element.find_by_attribute("id", "nested")
        self.assertEqual(str(found_nested), '<span id="nested"></span>')

    def test_get_attributes(self):
        """Test the get_attributes() method."""
        element = HTMLElement(
            tag="div", id="my-div", class_name="container", style="color: red"
        )
        self.assertEqual(
            element.get_attributes("id", "class_name"),
            {"id": "my-div", "class_name": "container"},
        )
        self.assertEqual(
            element.get_attributes(),
            {"id": "my-div", "class_name": "container", "style": "color: red"},
        )

    def test_count_children(self):
        """Test the count_children method."""
        element = HTMLElement(tag="div")
        element.append(HTMLElement(tag="span"), HTMLElement(tag="p"))
        self.assertEqual(element.count_children(), 2)

    def test_replace_child(self):
        """Test the replace_child() method."""
        element = HTMLElement(tag="div")
        element.append(HTMLElement(tag="span"), HTMLElement(tag="p"))
        self.assertEqual(element.count_children(), 2)
        self.assertEqual(str(element.children[0]), "<span></span>")
        self.assertEqual(str(element.children[1]), "<p></p>")
        element.replace_child(0, HTMLElement(tag="h1"))
        self.assertEqual(element.count_children(), 2)
        self.assertEqual(str(element.children[0]), "<h1></h1>")

    def test_callbacks(self):
        """Test the callback methods."""

        class TestElement(HTMLElement):
            def on_load(self):
                self.loaded = True

            def on_before_render(self):
                self.before_render_called = True

            def on_after_render(self):
                self.after_render_called = True

        element = TestElement(tag="div")
        self.assertTrue(hasattr(element, "loaded"))
        self.assertTrue(element.loaded)

        element.render()
        self.assertTrue(hasattr(element, "before_render_called"))
        self.assertTrue(element.before_render_called)

        self.assertTrue(hasattr(element, "after_render_called"))
        self.assertTrue(element.after_render_called)

    def test_add_attributes(self):
        """Test the add_attributes method."""
        element = HTMLElement(tag="div")
        element.add_attributes([("id", "my-div"), ("class", "container")])
        self.assertEqual(element.attributes, {"id": "my-div", "class": "container"})

        # Test overwriting existing attributes
        element.add_attributes([("id", "new-id"), ("style", "color: red")])
        self.assertEqual(
            element.attributes,
            {"id": "new-id", "class": "container", "style": "color: red"},
        )

    def test_to_dict(self):
        el1 = HTMLElement(tag="span", text="Child 1")
        el1.append(HTMLElement(tag="span", text="Child 2"))
        should_be = {
            "tag": "span",
            "self_closing": False,
            "attributes": {"text": "Child 1"},
            "text": "",
            "children": [
                {
                    "tag": "span",
                    "self_closing": False,
                    "attributes": {"text": "Child 2"},
                    "text": "",
                    "children": [],
                }
            ],
        }
        self.assertIsNotNone(el1.to_dict())
        self.assertIsInstance(el1.to_dict(), dict)
        self.assertEqual(el1.to_dict(), should_be)


if __name__ == "__main__":
    unittest.main()



from ydnatl.core.element import HTMLElement


class Image(HTMLElement):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **{**kwargs, "tag": "img", "self_closing": True})


class Video(HTMLElement):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **{**kwargs, "tag": "video"})


class Audio(HTMLElement):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **{**kwargs, "tag": "audio"})


class Source(HTMLElement):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **{**kwargs, "tag": "source", "self_closing": True})


class Picture(HTMLElement):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **{**kwargs, "tag": "picture"})


class Figure(HTMLElement):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **{**kwargs, "tag": "figure"})


class Figcaption(HTMLElement):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **{**kwargs, "tag": "figcaption"})


class Canvas(HTMLElement):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **{**kwargs, "tag": "canvas"})



from ydnatl.core.element import HTMLElement


class HTML(HTMLElement):
    def __init__(self, *args, **kwargs):
        super().__init__(
            *args,
            **{
                **kwargs | {"lang": "en", "dir": "ltr"},
                "tag": "html",
                "self_closing": False
            },
        )
        self._prefix = "<!DOCTYPE html>"


class Head(HTMLElement):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **{**kwargs, "tag": "head"})


class Body(HTMLElement):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **{**kwargs, "tag": "body"})


class Title(HTMLElement):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **{**kwargs, "tag": "title"})


class Meta(HTMLElement):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **{**kwargs, "tag": "meta", "self_closing": True})


class Link(HTMLElement):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **{**kwargs, "tag": "link", "self_closing": True})


class Script(HTMLElement):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **{**kwargs, "tag": "script"})


class Style(HTMLElement):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **{**kwargs, "tag": "style"})


class IFrame(HTMLElement):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **{**kwargs, "tag": "iframe"})



from ydnatl.core.element import HTMLElement


class UnorderedList(HTMLElement):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **{**kwargs, "tag": "ul"})

    @classmethod
    def with_items(cls, *items, **kwargs):
        ul = cls(**kwargs)
        for item in items:
            if isinstance(item, HTMLElement):
                ul.append(item)
            else:
                ul.append(ListItem(item))
        return ul


class OrderedList(HTMLElement):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **{**kwargs, "tag": "ol"})

    @classmethod
    def with_items(cls, *items, **kwargs):
        ol = cls(**kwargs)
        for item in items:
            if isinstance(item, HTMLElement):
                ol.append(item)
            else:
                ol.append(ListItem(item))
        return ol


class ListItem(HTMLElement):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **{**kwargs, "tag": "li"})


class Datalist(HTMLElement):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **{**kwargs, "tag": "datalist"})


class DescriptionDetails(HTMLElement):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **{**kwargs, "tag": "dd"})


class DescriptionList(HTMLElement):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **{**kwargs, "tag": "dl"})


class DescriptionTerm(HTMLElement):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **{**kwargs, "tag": "dt"})



from ydnatl.core.element import HTMLElement


class H1(HTMLElement):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **{**kwargs, "tag": "h1"})


class H2(HTMLElement):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **{**kwargs, "tag": "h2"})


class H3(HTMLElement):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **{**kwargs, "tag": "h3"})


class H4(HTMLElement):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **{**kwargs, "tag": "h4"})


class H5(HTMLElement):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **{**kwargs, "tag": "h5"})


class H6(HTMLElement):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **{**kwargs, "tag": "h6"})


class Paragraph(HTMLElement):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **{**kwargs, "tag": "p"})


class Blockquote(HTMLElement):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **{**kwargs, "tag": "blockquote"})


class Pre(HTMLElement):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **{**kwargs, "tag": "pre"})


class Quote(HTMLElement):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **{**kwargs, "tag": "q"})


class Cite(HTMLElement):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **{**kwargs, "tag": "cite"})


class Em(HTMLElement):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **{**kwargs, "tag": "em"})


class Italic(HTMLElement):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **{**kwargs, "tag": "i"})


class Span(HTMLElement):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **{**kwargs, "tag": "span"})


class Strong(HTMLElement):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **{**kwargs, "tag": "strong"})


class Abbr(HTMLElement):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **{**kwargs, "tag": "abbr"})


class Link(HTMLElement):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **{**kwargs, "tag": "a"})


class Small(HTMLElement):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **{**kwargs, "tag": "small"})


class Superscript(HTMLElement):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **{**kwargs, "tag": "sup"})


class Subscript(HTMLElement):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **{**kwargs, "tag": "sub"})


class Time(HTMLElement):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **{**kwargs, "tag": "time"})


class Code(HTMLElement):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **{**kwargs, "tag": "code"})



import uuid
import copy
import os
import functools

from typing import Callable, Any, Iterator, Union, List


class HTMLElement:
    __slots__ = ["_tag", "_children", "_text", "_attributes", "_self_closing"]

    def __init__(
        self,
        *children: Union["HTMLElement", str, List[Any]],
        tag: str,
        self_closing: bool = False,
        **attributes: str,
    ):
        PRESERVE_UNDERSCORE = {"class_name"}
        
        if not tag:
            raise ValueError("A valid HTML tag name is required")
        
        fixed_attributes = {(k if k in PRESERVE_UNDERSCORE else k.replace("_", "-")): v for k, v in attributes.items()}

        self._tag: str = tag
        self._children: List[HTMLElement] = []
        self._text: str = ""
        self._attributes: dict = fixed_attributes
        self._self_closing: bool = self_closing

        if os.environ.get("YDNATL_GENERATE_IDS"):
            self.generate_id()

        for child in self._flatten(children):
            self._add_child(child)

        self.on_load()

    def __str__(self) -> str:
        return self.render()
    
    def __del__(self) -> None:
        self.on_unload()

    @staticmethod
    def _flatten(items: Union[List[Any], tuple]) -> Iterator[Any]:
        """Recursively flattens nested iterables of children."""
        for item in items:
            if isinstance(item, (list, tuple)):
                yield from HTMLElement._flatten(item)
            else:
                yield item

    def _add_child(self, child: Union["HTMLElement", str]) -> None:
        """Adds a single child to the element."""
        if isinstance(child, HTMLElement):
            self._children.append(child)
        elif isinstance(child, str):
            self._text += child

    def prepend(self, *children: Union["HTMLElement", str, List[Any]]) -> None:
        """Prepends children to the current tag."""
        new_children: List[HTMLElement] = []
        for child in self._flatten(children):
            if isinstance(child, HTMLElement):
                new_children.append(child)
            elif isinstance(child, str):
                self._text = child + self._text
            else:
                raise ValueError(f"Invalid child type: {child}")
        self._children = new_children + self._children

    def append(self, *children: Union["HTMLElement", str, List[Any]]) -> None:
        """Appends children to the current tag."""
        for child in self._flatten(children):
            self._add_child(child)

    def filter(
        self, condition: Callable[[Any], bool], recursive: bool = False
    ) -> Iterator["HTMLElement"]:
        """Yields children (and optionally descendants) that meet the condition."""
        for child in self._children:
            if condition(child):
                yield child
            if recursive:
                yield from child.filter(condition, recursive=True)

    def remove_all(self, condition: Callable[[Any], bool]) -> None:
        """Removes all children that meet the condition."""
        to_remove = list(self.filter(condition))
        for child in to_remove:
            if child in self._children:
                self._children.remove(child)

    def clear(self) -> None:
        """Clears all children from the tag."""
        self._children.clear()

    def pop(self, index: int = 0) -> "HTMLElement":
        """Pops a child from the tag."""
        return self._children.pop(index)

    def first(self) -> Union["HTMLElement", None]:
        """Returns the first child of the tag."""
        return self._children[0] if self._children else None

    def last(self) -> Union["HTMLElement", None]:
        """Returns the last child of the tag."""
        return self._children[-1] if self._children else None

    def add_attribute(self, key: str, value: str) -> None:
        """Adds an attribute to the current tag."""
        self._attributes[key] = value
        
    def add_attributes(self, attributes: list[tuple[str, str]]) -> None:
        """Adds multiple attributes to the current tag."""
        for key, value in attributes:
            self._attributes[key] = value
        
    def remove_attribute(self, key: str) -> None:
        """Removes an attribute from the current tag."""
        self._attributes.pop(key, None)

    def get_attribute(self, key: str) -> Union[str, None]:
        """Gets an attribute from the current tag."""
        return self._attributes.get(key)

    def has_attribute(self, key: str) -> bool:
        """Checks if an attribute exists in the current tag."""
        return key in self._attributes

    def generate_id(self) -> None:
        """Generates an id for the current tag if not already present."""
        if "id" not in self._attributes:
            self._attributes["id"] = f"el-{str(uuid.uuid4())[:6]}"

    def clone(self) -> "HTMLElement":
        """Clones the current tag."""
        return copy.deepcopy(self)
    
    def replace_child(self, old_index: int, new_child: "HTMLElement") -> None:
        """Replaces a existing child element with a new child element."""
        self._children[old_index] = new_child

    def find_by_attribute(self, attr_name: str, attr_value: Any) -> Union["HTMLElement", None]:
        """Finds a child by an attribute."""
        def _find(element: "HTMLElement") -> Union["HTMLElement", None]:
            if element.get_attribute(attr_name) == attr_value:
                return element
            for child in element._children:
                result = _find(child)
                if result:
                    return result
            return None

        return _find(self)

    def get_attributes(self, *keys: str) -> dict:
        """Returns the attributes of the current tag."""
        if keys:
            return {key: self._attributes.get(key) for key in keys}
        return self._attributes

    def count_children(self) -> int:
        """Returns the number of children in the current tag."""
        return len(self._children)

    def on_load(self) -> None:
        """Callback called when the tag is loaded."""
        pass

    def on_before_render(self) -> None:
        """Callback called before the tag is rendered."""
        pass

    def on_after_render(self) -> None:
        """Callback called after the tag is rendered."""
        pass

    def on_unload(self) -> None:
        """Callback called when the tag is unloaded."""
        pass

    @property
    def tag(self) -> str:
        return self._tag

    @tag.setter
    def tag(self, value: str) -> None:
        self._tag = value

    @property
    def children(self) -> List["HTMLElement"]:
        return self._children

    @children.setter
    def children(self, value: List["HTMLElement"]) -> None:
        self._children = value

    @property
    def text(self) -> str:
        return self._text

    @text.setter
    def text(self, value: str) -> None:
        self._text = value

    @property
    def attributes(self) -> dict:
        return self._attributes

    @attributes.setter
    def attributes(self, value: dict) -> None:
        self._attributes = value

    @property
    def self_closing(self) -> bool:
        return self._self_closing

    @self_closing.setter
    def self_closing(self, value: bool) -> None:
        self._self_closing = value

    def _render_attributes(self) -> str:
        """Returns a string of HTML attributes for the tag."""
        attr_str = " ".join(
            f'{("class" if k == "class_name" else k)}="{v}"'
            for k, v in self._attributes.items()
        )
        return f" {attr_str}" if attr_str else ""

    @functools.lru_cache(maxsize=None)
    def render(self) -> str:
        """Renders the HTML element and its children to a string."""
        self.on_before_render()
        attributes = self._render_attributes()
        tag_start = f"<{self._tag}{attributes}"

        if self._self_closing:
            result = f"{tag_start} />"
        else:
            children_html = "".join(child.render() for child in self._children)
            result = f"{tag_start}>{self._text}{children_html}</{self._tag}>"

        if hasattr(self, "_prefix") and self._prefix:
            result = f"{self._prefix}{result}"

        self.on_after_render()
        return result
    
    @functools.lru_cache(maxsize=None)
    def to_dict(self) -> dict:
        return {
            "tag": self._tag,
            "self_closing": self._self_closing,
            "attributes": self._attributes.copy(),
            "text": self._text,
            "children": list(map(lambda child: child.to_dict(), self._children))
        }


import unittest

from ydnatl.tags.html import (
    HTML,
    Head,
    Body,
    Title,
    Meta,
    Link,
    Script,
    Style,
    IFrame,
)
from ydnatl.core.element import HTMLElement


class TestHTMLTags(unittest.TestCase):

    def test_head(self):
        """Test the creation of an empty head element."""
        head = Head()
        self.assertEqual(head.tag, "head")
        self.assertEqual(str(head), "<head></head>")

    def test_body(self):
        """Test the creation of an empty body element."""
        body = Body()
        self.assertEqual(body.tag, "body")
        self.assertEqual(str(body), "<body></body>")

    def test_title(self):
        """Test the creation of a title element with text content."""
        title = Title("My Page")
        self.assertEqual(title.tag, "title")
        self.assertEqual(str(title), "<title>My Page</title>")

    def test_meta(self):
        """Test the creation of an empty meta element."""
        meta = Meta()
        self.assertEqual(meta.tag, "meta")
        self.assertEqual(str(meta), "<meta />")

    def test_link(self):
        """Test the creation of an empty link element."""
        link = Link()
        self.assertEqual(link.tag, "link")
        self.assertEqual(str(link), "<link />")

    def test_script(self):
        """Test the creation of an empty script element."""
        script = Script()
        self.assertEqual(script.tag, "script")
        self.assertEqual(str(script), "<script></script>")

    def test_style(self):
        """Test the creation of an empty style element."""
        style = Style()
        self.assertEqual(style.tag, "style")
        self.assertEqual(str(style), "<style></style>")

    def test_iframe(self):
        """Test the creation of an empty iframe element."""
        iframe = IFrame()
        self.assertEqual(iframe.tag, "iframe")
        self.assertEqual(str(iframe), "<iframe></iframe>")

    def test_html(self):
        """Test the creation of an empty HTML document."""
        html = HTML()
        self.assertEqual(html.tag, "html")
        expected = "<!DOCTYPE html><html lang=\"en\" dir=\"ltr\"></html>"
        self.assertEqual(str(html), expected)

    def test_inheritance(self):
        """Test that all HTML-related classes inherit from HTMLElement."""
        for cls in [HTML, Head, Body, Title, Meta, Link, Script, Style, IFrame]:
            self.assertTrue(issubclass(cls, HTMLElement))


if __name__ == "__main__":
    unittest.main()



from ydnatl.core.element import HTMLElement


class Textarea(HTMLElement):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **{**kwargs, "tag": "textarea"})


class Select(HTMLElement):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **{**kwargs, "tag": "select"})

    @staticmethod
    def with_items(*items, **kwargs):
        opt = Select(**kwargs)
        for item in items:
            if isinstance(item, HTMLElement):
                opt.append(item)
            else:
                opt.append(Option(item))
        return opt


class Option(HTMLElement):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **{**kwargs, "tag": "option"})


class Button(HTMLElement):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **{**kwargs, "tag": "button"})


class Fieldset(HTMLElement):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **{**kwargs, "tag": "fieldset"})


class Form(HTMLElement):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **{**kwargs, "tag": "form"})

    @staticmethod
    def with_fields(*items, **kwargs):
        form = Form(**kwargs)
        for item in items:
            form.append(item)  # TODO: Check if item is a valid field class
        return form


class Input(HTMLElement):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **{**kwargs, "tag": "input", "self_closing": True})


class Label(HTMLElement):
    def __init__(self, *args, **kwargs):
        if 'for_element' in kwargs:
            kwargs['for'] = kwargs.pop('for_element')
        super().__init__(*args, **{**kwargs, "tag": "label"})
        
        
class Optgroup(HTMLElement):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **{**kwargs, "tag": "optgroup"})



from ydnatl.core.element import HTMLElement


class Div(HTMLElement):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **{**kwargs, "tag": "div"})


class Section(HTMLElement):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **{**kwargs, "tag": "section"})


class Header(HTMLElement):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **{**kwargs, "tag": "header"})


class Nav(HTMLElement):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **{**kwargs, "tag": "nav"})


class Footer(HTMLElement):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **{**kwargs, "tag": "footer"})


class HorizontalRule(HTMLElement):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **{**kwargs, "tag": "hr", "self_closing": True})


class Main(HTMLElement):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **{**kwargs, "tag": "main"})
